@@ -1,11 +1,13 @@
 import logging
 from copy import deepcopy
 from dataclasses import dataclass
+from time import sleep
 from typing import List
 
 import requests
 
-LOGGER = logging.getLogger(__name__)
+LOGGER = logging.getLogger("corganizeclient")
+logging.basicConfig()
 
 
 @dataclass(frozen=True)
@@ -20,42 +22,60 @@ class CorganizeClient:
     def _compose_url(self, resource):
         return "/".join([s.strip("/") for s in (self.host, resource)])
 
-    def get_recently_modified_files(self, limit):
+    def get_recently_modified_files(self, **kwargs):
         url = self._compose_url("/files")
-        return self._get_paginated_files(url, limit=limit)
+        return self._get_paginated_files(url, **kwargs)
 
-    def get_least_recently_modified_files(self, limit):
+    def get_least_recently_modified_files(self, **kwargs):
         url = self._compose_url("/files")
         headers = {**self._default_headers, "order": "asc"}
-        return self._get_paginated_files(url, headers=headers, limit=limit)
+        return self._get_paginated_files(url, headers=headers, **kwargs)
 
-    def get_active_files(self, limit):
+    def get_active_files(self, **kwargs):
         url = self._compose_url("/files/active")
-        return self._get_paginated_files(url, limit=limit)
+        return self._get_paginated_files(url, **kwargs)
 
-    def get_least_recent_active_files(self, limit):
+    def get_least_recent_active_files(self, **kwargs):
         url = self._compose_url("/files/active")
         headers = {**self._default_headers, "order": "asc"}
-        return self._get_paginated_files(url, headers=headers, limit=limit)
+        return self._get_paginated_files(url, headers=headers, **kwargs)
 
-    def get_stale_files(self, limit):
+    def get_stale_files(self, **kwargs):
         url = self._compose_url("/files/stale")
-        return self._get_paginated_files(url, limit=limit)
+        return self._get_paginated_files(url, **kwargs)
 
-    def get_incomplete_files(self, limit):
+    def get_incomplete_files(self, **kwargs):
         url = self._compose_url("/files/incomplete")
-        return self._get_paginated_files(url, limit=limit)
+        return self._get_paginated_files(url, **kwargs)
 
-    def create_files(self, files: List[dict]):
-        assert isinstance(files, list)
+    def create_files(self, files: List[dict], chunk_size: int = 50, interval: int = 15):
+        url = self._compose_url("/files")
+        remaining_files = files
+        result = dict(
+            created=[],
+            skipped=[],
+            failed=[]
+        )
 
-        url = self._compose_url("/files/bulk")
-        r = requests.post(url, json=files, headers=self._default_headers)
+        while True:
+            chunk = remaining_files[:chunk_size]
+            r = requests.post(url, json=chunk, headers=self._default_headers)
 
-        if not r.ok:
-            raise RuntimeError(r.text)
+            if not r.ok:
+                result["failed"] += [f["fileid"] for f in chunk]
 
-        return r.json()
+            chunk_result = r.json()
+            result["created"] += chunk_result["created"]
+            result["skipped"] += chunk_result["skipped"]
+
+            remaining_files = remaining_files[chunk_size:]
+            if len(remaining_files) == 0:
+                break
+
+            LOGGER.debug(f"Sleeping for {interval=} seconds; {len(remaining_files)=}")
+            sleep(interval)
+
+        return result
 
     def update_file(self, file):
         assert isinstance(file, dict)
@@ -79,7 +99,7 @@ class CorganizeClient:
         r.raise_for_status()
         return r.json()
 
-    def _get_paginated_files(self, url: str, headers: dict = None, limit: int = 1000):
+    def _get_paginated_files(self, url: str, headers: dict = None, limit: int = 1000, interval: int = 0):
         return_files = list()
 
         if not headers:
@@ -110,6 +130,7 @@ class CorganizeClient:
             })
 
             LOGGER.info("next_token found")
+            sleep(interval)
 
         LOGGER.info("End of pagination")
 
